import router from './router'
import { ElMessage } from 'element-plus'
import NProgress from 'nprogress'
import 'nprogress/nprogress.css'
import { getToken } from '@/utils/auth'
import { isHttp, isPathMatch } from '@/utils/validate'
import { isRelogin } from '@/utils/request'
import useUserStore from '@/store/modules/user'
import useSettingsStore from '@/store/modules/settings'
import usePermissionStore from '@/store/modules/permission'
import axios from 'axios'

NProgress.configure({ showSpinner: false })

const whiteList = ['/login', '/register']

const isWhiteList = (path) => {
  return whiteList.some((pattern) => isPathMatch(pattern, path))
}

// 后端登录接口
const loginBySSOApi = '/auth/loginBySSO'
const unifiedLoginRoute = '/login'
// 登录路由
const loginRoute =  import.meta.env.VITE_APP_LOGIN_ROUTE

router.beforeEach((to, from, next) => {
  NProgress.start()
  if (getToken()) {
    to.meta.title && useSettingsStore().setTitle(to.meta.title)
    /* has token*/
    if (to.path === '/login') {
      next({ path: '/' })
      NProgress.done()
    } else if (isWhiteList(to.path)) {
      next()
    } else {
      if (useUserStore().roles.length === 0) {
        isRelogin.show = true
        // 判断当前用户是否已拉取完user_info信息
        useUserStore()
          .getInfo()
          .then(() => {
            isRelogin.show = false
            usePermissionStore()
              .generateRoutes()
              .then((accessRoutes) => {
                // 根据roles权限生成可访问的路由表
                accessRoutes.forEach((route) => {
                  if (!isHttp(route.path)) {
                    router.addRoute(route) // 动态添加可访问路由表
                  }
                })
                next({ ...to, replace: true }) // hack方法 确保addRoutes已完成
              })
          })
          .catch((err) => {
            useUserStore()
              .logOut()
              .then(() => {
                ElMessage.error(err)
                next({ path: '/' })
              })
          })
      } else {
        next()
      }
    }
  }

  // //若依系统登录
  // else {
  //   // 没有token
  //   if (isWhiteList(to.path)) {
  //     // 在免登录白名单，直接进入
  //     next()
  //   } else {
  //     next(`/login?redirect=${to.fullPath}`) // 否则全部重定向到登录页
  //     NProgress.done()
  //   }
  // }

   //iam系统登录
  else {
    // 没有token
    if (isWhiteList(to.path)) {
      // 在免登录白名单，直接进入
      if (to.path === unifiedLoginRoute) {
        // 从URL中截取code参数
        const urlParams = new URLSearchParams(window.location.search)
        const code = urlParams.get('code')
        if (code) {
          // 请求后端/auth/loginBySSO接口
          axios
            .post(loginBySSOApi, { ticket: code })
            .then((response) => {
              if (response.data.success) {
                // 登录成功，跳转到首页
                next({ path: '/' })
              } else {
                ElMessage.error('登录失败，请重试')
                next()
              }
            })
            .catch((error) => {
              ElMessage.error('登录请求出错，请重试')
              next()
            })
        } else {
          next()
        }
      } else {
        next()
      }
    } else {
      // 跳转到IAM登录页面
      window.location.href = loginRoute
      NProgress.done()
    }
  }

})

router.afterEach(() => {
  NProgress.done()
})
